@@ -1,6 +1,5 @@
 """Provides the main() routine and ColaApplication"""
 # pylint: disable=unused-import
-from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 import argparse
 import os
@@ -8,10 +7,6 @@ import signal
 import sys
 import time
 
-__copyright__ = """
-Copyright (C) 2007-2022 David Aguilar and contributors
-"""
-
 try:
     from qtpy import QtCore
 except ImportError as error:
@@ -166,7 +161,7 @@ def get_icon_themes(context):
 
 
 # style note: we use camelCase here since we're masquerading a Qt class
-class ColaApplication(object):
+class ColaApplication:
     """The main cola application
 
     ColaApplication handles i18n of user-visible data
@@ -195,11 +190,15 @@ class ColaApplication(object):
         theme = themes.find_theme(theme_str)
         self.theme = theme
         self._app.setStyleSheet(theme.build_style_sheet(self._app.palette()))
-        if theme_str != 'default':
+
+        is_macos_theme = theme_str.startswith('macos-')
+        if is_macos_theme:
+            themes.apply_platform_theme(theme_str)
+        elif theme_str != 'default':
             self._app.setPalette(theme.build_palette(self._app.palette()))
 
     def _install_hidpi_config(self):
-        """Sets QT HIDPI scalling (requires Qt 5.6)"""
+        """Sets QT HiDPI scaling (requires Qt 5.6)"""
         value = self.context.cfg.get('cola.hidpi', default=hidpi.Option.AUTO)
         hidpi.apply_choice(value)
 
@@ -247,7 +246,7 @@ class ColaQApplication(QtWidgets.QApplication):
     """QApplication implementation for handling custom events"""
 
     def __init__(self, context, argv):
-        super(ColaQApplication, self).__init__(argv)
+        super().__init__(argv)
         self.context = context
         # Make icons sharp in HiDPI screen
         if hasattr(Qt, 'AA_UseHighDpiPixmaps'):
@@ -263,7 +262,7 @@ class ColaQApplication(QtWidgets.QApplication):
                     'cola.refreshonfocus', default=False
                 ):
                     cmds.do(cmds.Refresh, context)
-        return super(ColaQApplication, self).event(e)
+        return super().event(e)
 
     def commitData(self, session_mgr):
         """Save session data"""
@@ -274,7 +273,7 @@ class ColaQApplication(QtWidgets.QApplication):
             return
         sid = session_mgr.sessionId()
         skey = session_mgr.sessionKey()
-        session_id = '%s_%s' % (sid, skey)
+        session_id = f'{sid}_{skey}'
         session = Session(session_id, repo=core.getcwd())
         session.update()
         view.save_state(settings=session)
@@ -508,7 +507,6 @@ def async_update(context):
     """Update the model in the background
 
     git-cola should startup as quickly as possible.
-
     """
     update_status = partial(context.model.update_status, update_index=True)
     task = qtutils.SimpleTask(update_status)
@@ -553,7 +551,7 @@ def initialize():
             os.chdir('..')
 
 
-class Timer(object):
+class Timer:
     """Simple performance timer"""
 
     def __init__(self):
@@ -578,10 +576,10 @@ class Timer(object):
     def display(self, key):
         """Display a timer"""
         elapsed = self.elapsed(key)
-        sys.stdout.write('%s: %.5fs\n' % (key, elapsed))
+        sys.stdout.write(f'{key}: {elapsed:.5f}s\n')
 
 
-class NullArgs(object):
+class NullArgs:
     """Stub arguments for interactive API use"""
 
     def __init__(self):
@@ -595,7 +593,7 @@ def null_args():
     return NullArgs()
 
 
-class ApplicationContext(object):
+class ApplicationContext:
     """Context for performing operations on Git and related data models"""
 
     def __init__(self, args):
