@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 import errno
 import functools
 
@@ -41,7 +40,7 @@ def memoize(func):
     """
     A decorator for memoizing function calls
 
-    http://en.wikipedia.org/wiki/Memoization
+    https://en.wikipedia.org/wiki/Memoization
 
     """
     func.cache = {}
@@ -66,9 +65,9 @@ def _memoize(func, *args, **opts):
 def interruptable(func, *args, **opts):
     """Handle interruptible system calls
 
-    OSX and others are known to interrupt system calls
+    macOS and others are known to interrupt system calls
 
-        http://en.wikipedia.org/wiki/PCLSRing
+        https://en.wikipedia.org/wiki/PCLSRing
         http://en.wikipedia.org/wiki/Unix_philosophy#Worse_is_better
 
     The @interruptable decorator handles this situation
@@ -77,7 +76,7 @@ def interruptable(func, *args, **opts):
     while True:
         try:
             result = func(*args, **opts)
-        except (IOError, OSError) as e:
+        except OSError as e:
             if e.errno in (errno.EINTR, errno.EINVAL):
                 continue
             raise e
