@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2017 David Aguilar
+# Copyright (C) 2008-2024 David Aguilar
 # Copyright (C) 2015 Daniel Harding
 """Filesystem monitor for Linux and Windows
 
@@ -6,7 +6,6 @@ Linux monitoring uses using inotify.
 Windows monitoring uses pywin32 and the ReadDirectoryChanges function.
 
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import errno
 import os
 import os.path
@@ -26,16 +25,21 @@ from .interaction import Interaction
 
 AVAILABLE = None
 
+pywintypes = None
+win32file = None
+win32con = None
+win32event = None
 if utils.is_win32():
     try:
         import pywintypes
         import win32con
         import win32event
         import win32file
+
+        AVAILABLE = 'pywin32'
     except ImportError:
         pass
-    else:
-        AVAILABLE = 'pywin32'
+
 elif utils.is_linux():
     try:
         from . import inotify
@@ -223,7 +227,7 @@ if AVAILABLE == 'inotify':
                 try:
                     events = poll_obj.poll(timeout)
                 # pylint: disable=duplicate-except
-                except (OSError, select.error):
+                except OSError:
                     continue
                 else:
                     if not self._running:
@@ -289,7 +293,7 @@ if AVAILABLE == 'inotify':
                     inotify.rm_watch(self._inotify_fd, wd)
                 except OSError as e:
                     if e.errno == errno.EINVAL:
-                        # This error can occur if the target of the wd was
+                        # This error can occur if the target of the watch was
                         # removed on the filesystem before we call
                         # inotify.rm_watch() so ignore it.
                         continue
@@ -354,7 +358,7 @@ if AVAILABLE == 'inotify':
 
 if AVAILABLE == 'pywin32':
 
-    class _Win32Watch(object):
+    class _Win32Watch:
         def __init__(self, path, flags):
             self.flags = flags
 
@@ -377,16 +381,24 @@ if AVAILABLE == 'pywin32':
                 self.overlapped = pywintypes.OVERLAPPED()
                 self.overlapped.hEvent = self.event
                 self._start()
-            except Exception:
+            except Exception:  # pylint: disable=broad-exception-caught,broad-except
                 self.close()
-                raise
+
+        def append(self, events):
+            """Append our event to the events list when valid"""
+            if self.event is not None:
+                events.append(self.event)
 
         def _start(self):
+            if self.handle is None:
+                return
             win32file.ReadDirectoryChangesW(
                 self.handle, self.buffer, True, self.flags, self.overlapped
             )
 
         def read(self):
+            if self.handle is None or self.event is None:
+                return []
             if win32event.WaitForSingleObject(self.event, 0) == win32event.WAIT_TIMEOUT:
                 result = []
             else:
@@ -440,10 +452,10 @@ if AVAILABLE == 'pywin32':
 
                 if self._worktree is not None:
                     self._worktree_watch = _Win32Watch(self._worktree, self._FLAGS)
-                    events.append(self._worktree_watch.event)
+                    self._worktree_watch.append(events)
 
                 self._git_dir_watch = _Win32Watch(self._git_dir, self._FLAGS)
-                events.append(self._git_dir_watch.event)
+                self._git_dir_watch.append(events)
 
                 self._log_enabled_message()
 
