@@ -1,4 +1,3 @@
-# -* coding: utf-8 -*-
 #
 # License: MIT (see extras/polib/LICENSE file provided)
 # vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4:
@@ -12,7 +11,6 @@ modify entries, comments or metadata, etc. or create new po files from scratch.
 **polib** provides a simple and pythonic API via the :func:`~polib.pofile` and
 :func:`~polib.mofile` convenience functions.
 """
-from __future__ import absolute_import, division, print_function
 import array
 import codecs
 import os
@@ -47,25 +45,16 @@ default_encoding = 'utf-8'
 # python 2/3 compatibility helpers {{{
 
 
-if sys.version_info < (3,):
-    PY3 = False
-    text_type = compat.ustr
+PY3 = True
+text_type = str
 
-    def b(s):
-        return s
 
-    def u(s):
-        return compat.ustr(s, "unicode_escape")
+def b(s):
+    return s.encode('utf-8')
 
-else:
-    PY3 = True
-    text_type = str
 
-    def b(s):
-        return s.encode("utf-8")
-
-    def u(s):
-        return s
+def u(s):
+    return s
 
 
 # }}}
@@ -265,7 +254,7 @@ def escape(st):
         .replace('\t', r'\t')
         .replace('\r', r'\r')
         .replace('\n', r'\n')
-        .replace('\"', r'\"')
+        .replace('"', r'\"')
     )
 
 
@@ -423,7 +412,7 @@ class _BaseFile(list):
         # But if pickling, we never want to check for duplicates anyway.
         if getattr(self, 'check_for_duplicates', False) and entry in self:
             raise ValueError('Entry "%s" already exists' % entry.msgid)
-        super(_BaseFile, self).append(entry)
+        super().append(entry)
 
     def insert(self, index, entry):
         """
@@ -441,7 +430,7 @@ class _BaseFile(list):
         """
         if self.check_for_duplicates and entry in self:
             raise ValueError('Entry "%s" already exists' % entry.msgid)
-        super(_BaseFile, self).insert(index, entry)
+        super().insert(index, entry)
 
     def metadata_as_entry(self):
         """
@@ -453,7 +442,7 @@ class _BaseFile(list):
             strs = []
             for name, value in mdata:
                 # Strip whitespace off each line in a multi-line entry
-                strs.append('%s: %s' % (name, value))
+                strs.append(f'{name}: {value}')
             e.msgstr = '\n'.join(strs) + '\n'
         if self.metadata_is_fuzzy:
             e.flags.append('fuzzy')
@@ -477,14 +466,14 @@ class _BaseFile(list):
             string, controls how universal newlines works
         """
         if self.fpath is None and fpath is None:
-            raise IOError('You must provide a file path to save() method')
+            raise OSError('You must provide a file path to save() method')
         contents = getattr(self, repr_method)()
         if fpath is None:
             fpath = self.fpath
         if repr_method == 'to_binary':
             fhandle = open(fpath, 'wb')
         else:
-            fhandle = io.open(fpath, 'w', encoding=self.encoding, newline=newline)
+            fhandle = open(fpath, 'w', encoding=self.encoding, newline=newline)
             if not isinstance(contents, text_type):
                 contents = contents.decode(self.encoding)
         fhandle.write(contents)
@@ -633,7 +622,7 @@ class _BaseFile(list):
         offsets = koffsets + voffsets
 
         output = struct.pack(
-            "Iiiiiii",
+            'Iiiiiii',
             # Magic number
             MOFile.MAGIC,
             # Version
@@ -649,9 +638,9 @@ class _BaseFile(list):
             keystart,
         )
         if PY3 and sys.version_info.minor > 1:  # python 3.2 or superior
-            output += array.array("i", offsets).tobytes()
+            output += array.array('i', offsets).tobytes()
         else:
-            output += array.array("i", offsets).tostring()  # pylint: disable=no-member
+            output += array.array('i', offsets).tostring()  # pylint: disable=no-member
         output += ids
         output += strs
         return output
@@ -684,7 +673,7 @@ class POFile(_BaseFile):
         ret, headers = '', self.header.split('\n')
         for header in headers:
             if not header:
-                ret += "#\n"
+                ret += '#\n'
             elif header[:1] in [',', ':']:
                 ret += '#%s\n' % header
             else:
@@ -760,8 +749,8 @@ class POFile(_BaseFile):
             object POFile, the reference catalog.
         """
         # Store entries in dict/set for faster access
-        self_entries = dict((entry.msgid_with_context, entry) for entry in self)
-        refpot_msgids = set(entry.msgid_with_context for entry in refpot)
+        self_entries = {entry.msgid_with_context: entry for entry in self}
+        refpot_msgids = {entry.msgid_with_context for entry in refpot}
         # Merge entries that are in the refpot
         for entry in refpot:
             e = self_entries.get(entry.msgid_with_context)
@@ -856,7 +845,7 @@ class MOFile(_BaseFile):
 # class _BaseEntry {{{
 
 
-class _BaseEntry(object):
+class _BaseEntry:
     """
     Base class for :class:`~polib.POEntry` and :class:`~polib.MOEntry` classes.
     This class should **not** be instantiated directly.
@@ -907,13 +896,13 @@ class _BaseEntry(object):
         ret = []
         # write the msgctxt if any
         if self.msgctxt is not None:
-            ret += self._str_field("msgctxt", delflag, "", self.msgctxt, wrapwidth)
+            ret += self._str_field('msgctxt', delflag, '', self.msgctxt, wrapwidth)
         # write the msgid
-        ret += self._str_field("msgid", delflag, "", self.msgid, wrapwidth)
+        ret += self._str_field('msgid', delflag, '', self.msgid, wrapwidth)
         # write the msgid_plural if any
         if self.msgid_plural:
             ret += self._str_field(
-                "msgid_plural", delflag, "", self.msgid_plural, wrapwidth
+                'msgid_plural', delflag, '', self.msgid_plural, wrapwidth
             )
         if self.msgstr_plural:
             # write the msgstr_plural if any
@@ -924,11 +913,11 @@ class _BaseEntry(object):
                 msgstr = msgstrs[index]
                 plural_index = '[%s]' % index
                 ret += self._str_field(
-                    "msgstr", delflag, plural_index, msgstr, wrapwidth
+                    'msgstr', delflag, plural_index, msgstr, wrapwidth
                 )
         else:
             # otherwise write the msgstr
-            ret += self._str_field("msgstr", delflag, "", self.msgstr, wrapwidth)
+            ret += self._str_field('msgstr', delflag, '', self.msgstr, wrapwidth)
         ret.append('')
         ret = u('\n').join(ret)
         return ret
@@ -984,15 +973,15 @@ class _BaseEntry(object):
             # quick and dirty trick to get the real field name
             fieldname = fieldname[9:]
 
-        ret = ['%s%s%s "%s"' % (delflag, fieldname, plural_index, escape(lines.pop(0)))]
+        ret = [f'{delflag}{fieldname}{plural_index} "{escape(lines.pop(0))}"']
         for line in lines:
-            ret.append('%s"%s"' % (delflag, escape(line)))
+            ret.append(f'{delflag}"{escape(line)}"')
         return ret
 
     @property
     def msgid_with_context(self):
         if self.msgctxt:
-            return '%s%s%s' % (self.msgctxt, "\x04", self.msgid)
+            return '{}{}{}'.format(self.msgctxt, '\x04', self.msgid)
         return self.msgid
 
 
@@ -1066,14 +1055,14 @@ class POEntry(_BaseEntry):
                             break_long_words=False,
                         )
                     else:
-                        ret.append('%s%s' % (c[1], comment))
+                        ret.append(f'{c[1]}{comment}')
 
         # occurrences (with text wrapping as xgettext does)
         if not self.obsolete and self.occurrences:
             filelist = []
             for fpath, lineno in self.occurrences:
                 if lineno:
-                    filelist.append('%s:%s' % (fpath, lineno))
+                    filelist.append(f'{fpath}:{lineno}')
                 else:
                     filelist.append(fpath)
             filestr = ' '.join(filelist)
@@ -1102,13 +1091,13 @@ class POEntry(_BaseEntry):
         # previous context and previous msgid/msgid_plural
         fields = ['previous_msgctxt', 'previous_msgid', 'previous_msgid_plural']
         if self.obsolete:
-            prefix = "#~| "
+            prefix = '#~| '
         else:
-            prefix = "#| "
+            prefix = '#| '
         for f in fields:
             val = getattr(self, f)
             if val is not None:
-                ret += self._str_field(f, prefix, "", val, wrapwidth)
+                ret += self._str_field(f, prefix, '', val, wrapwidth)
 
         ret.append(_BaseEntry.__unicode__(self, wrapwidth))
         ret = u('\n').join(ret)
@@ -1281,7 +1270,7 @@ class MOEntry(_BaseEntry):
 # class _POFileParser {{{
 
 
-class _POFileParser(object):
+class _POFileParser:
     """
     A finite state machine to parse efficiently and correctly po
     file format.
@@ -1308,10 +1297,10 @@ class _POFileParser(object):
         enc = kwargs.get('encoding', default_encoding)
         if _is_file(pofile):
             try:
-                self.fhandle = io.open(pofile, 'rt', encoding=enc)
+                self.fhandle = open(pofile, encoding=enc)
             except LookupError:
                 enc = default_encoding
-                self.fhandle = io.open(pofile, 'rt', encoding=enc)
+                self.fhandle = open(pofile, encoding=enc)
         else:
             self.fhandle = pofile.splitlines()
 
@@ -1442,7 +1431,7 @@ class _POFileParser(object):
             if tokens[0] in keywords and nb_tokens > 1:
                 line = line[len(tokens[0]) :].lstrip()
                 if re.search(r'([^\\]|^)"', line[1:-1]):
-                    raise IOError(
+                    raise OSError(
                         'Syntax error in po file %s(line %s): '
                         'unescaped double quote found' % (fpath, self.current_line)
                     )
@@ -1461,7 +1450,7 @@ class _POFileParser(object):
             elif line[:1] == '"':
                 # we are on a continuation line
                 if re.search(r'([^\\]|^)"', line[1:-1]):
-                    raise IOError(
+                    raise OSError(
                         'Syntax error in po file %s(line %s): '
                         'unescaped double quote found' % (fpath, self.current_line)
                     )
@@ -1491,7 +1480,7 @@ class _POFileParser(object):
 
             elif tokens[0] == '#|':
                 if nb_tokens <= 1:
-                    raise IOError(
+                    raise OSError(
                         'Syntax error in po file %s(line %s)'
                         % (fpath, self.current_line)
                     )
@@ -1507,7 +1496,7 @@ class _POFileParser(object):
 
                 if nb_tokens == 2:
                     # Invalid continuation line.
-                    raise IOError(
+                    raise OSError(
                         'Syntax error in po file %s(line %s): '
                         'invalid continuation line' % (fpath, self.current_line)
                     )
@@ -1515,7 +1504,7 @@ class _POFileParser(object):
                 # we are on a "previous translation" comment line,
                 if tokens[1] not in prev_keywords:
                     # Unknown keyword in previous translation comment.
-                    raise IOError(
+                    raise OSError(
                         'Syntax error in po file %s(line %s): '
                         'unknown keyword %s' % (fpath, self.current_line, tokens[1])
                     )
@@ -1527,8 +1516,8 @@ class _POFileParser(object):
                 self.process(prev_keywords[tokens[1]])
 
             else:
-                raise IOError(
-                    'Syntax error in po file %s(line %s)' % (fpath, self.current_line)
+                raise OSError(
+                    f'Syntax error in po file {fpath}(line {self.current_line})'
                 )
 
         if self.current_entry and len(tokens) > 0 and not tokens[0].startswith('#'):
@@ -1597,9 +1586,7 @@ class _POFileParser(object):
             fpath = '%s ' % self.instance.fpath if self.instance.fpath else ''
             if hasattr(self.fhandle, 'close'):
                 self.fhandle.close()
-            raise IOError(
-                'Syntax error in po file %s(line %s)' % (fpath, self.current_line)
-            )
+            raise OSError(f'Syntax error in po file {fpath}(line {self.current_line})')
 
     # state handlers
 
@@ -1747,7 +1734,7 @@ class _POFileParser(object):
 # class _MOFileParser {{{
 
 
-class _MOFileParser(object):
+class _MOFileParser:
     """
     A class to parse binary mo files.
     """
@@ -1804,14 +1791,14 @@ class _MOFileParser(object):
         elif magic_number == MOFile.MAGIC_SWAPPED:
             ii = '>II'
         else:
-            raise IOError('Invalid mo file, magic number is incorrect !')
+            raise OSError('Invalid mo file, magic number is incorrect !')
         self.instance.magic_number = magic_number
         # parse the version number and the number of strings
         version, numofstrings = self._readbinary(ii, 8)
         # from MO file format specs: "A program seeing an unexpected major
         # revision number should stop reading the MO file entirely"
         if version >> 16 not in (0, 1):
-            raise IOError('Invalid mo file, unexpected major revision number')
+            raise OSError('Invalid mo file, unexpected major revision number')
         self.instance.version = version
         # original strings and translation strings hash table offset
         msgids_hash_offset, msgstrs_hash_offset = self._readbinary(ii, 8)
@@ -1852,9 +1839,7 @@ class _MOFileParser(object):
                 entry = self._build_entry(
                     msgid=msgid_tokens[0],
                     msgid_plural=msgid_tokens[1],
-                    msgstr_plural=dict(
-                        (k, v) for k, v in enumerate(msgstr.split(b('\0')))
-                    ),
+                    msgstr_plural=dict(enumerate(msgstr.split(b('\x00')))),
                 )
             else:
                 entry = self._build_entry(msgid=msgid, msgstr=msgstr)
