@@ -1,5 +1,3 @@
-# encoding: utf-8
-from __future__ import absolute_import, division, print_function, unicode_literals
 import platform
 import webbrowser
 import sys
@@ -32,19 +30,18 @@ class ExpandingTabBar(QtWidgets.QTabBar):
     The setExpanding(True) method does not work in practice because
     it respects the OS style.  We override the style by implementing
     tabSizeHint() so that we can specify the size explicitly.
-
     """
 
     def tabSizeHint(self, tab_index):
         width = self.parent().width() // max(2, self.count()) - 1
-        size = super(ExpandingTabBar, self).tabSizeHint(tab_index)
+        size = super().tabSizeHint(tab_index)
         size.setWidth(width)
         return size
 
 
 class ExpandingTabWidget(QtWidgets.QTabWidget):
     def __init__(self, parent=None):
-        super(ExpandingTabWidget, self).__init__(parent)
+        super().__init__(parent)
         self.setTabBar(ExpandingTabBar(self))
 
     def resizeEvent(self, event):
@@ -54,7 +51,7 @@ class ExpandingTabWidget(QtWidgets.QTabWidget):
         width = event.size().width()
         height = self.tabBar().height()
         self.tabBar().resize(width, height)
-        return super(ExpandingTabWidget, self).resizeEvent(event)
+        return super().resizeEvent(event)
 
 
 class AboutView(QtWidgets.QDialog):
@@ -126,7 +123,7 @@ def copyright_text():
     return """
 Git Cola: The highly caffeinated Git GUI
 
-Copyright (C) 2007-2023 David Aguilar and contributors
+Copyright (C) 2007-2024 David Aguilar and contributors
 
 This program is free software: you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -154,7 +151,11 @@ def version_text(context):
     qt_version = qtpy.QT_VERSION
     qtpy_version = qtpy.__version__
     pyqt_api_name = qtpy.API_NAME
-    if qtpy.PYQT5 or qtpy.PYQT4:
+    if (
+        getattr(qtpy, 'PYQT6', False)
+        or getattr(qtpy, 'PYQT5', False)
+        or getattr(qtpy, 'PYQT4', False)
+    ):
         pyqt_api_version = qtpy.PYQT_VERSION
     elif qtpy.PYSIDE:
         pyqt_api_version = qtpy.PYSIDE_VERSION
@@ -194,31 +195,12 @@ def version_text(context):
     )
 
 
-def link(url, text, palette=None):
-    if palette is None:
-        palette = QtGui.QPalette()
-
-    color = palette.color(QtGui.QPalette.Foreground)
-    rgb = 'rgb(%s, %s, %s)' % (color.red(), color.green(), color.blue())
-    scope = dict(rgb=rgb, text=text, url=url)
-
-    return (
-        """
-        <a style="font-style: italic; text-decoration: none; color: %(rgb)s;"
-            href="%(url)s">
-            %(text)s
-        </a>
-    """
-        % scope
-    )
-
-
 def mailto(email, text, palette):
-    return link('mailto:%s' % email, text, palette) + '<br>'
+    return qtutils.link('mailto:%s' % email, text, palette) + '<br>'
 
 
 def render_authors(authors):
-    """Render a list of author details into richtext html"""
+    """Render a list of author details into rich text html"""
     for x in authors:
         x.setdefault('email', '')
 
@@ -380,7 +362,7 @@ def authors_text():
         dict(name='Markus Heidelberg', title=N_('Developer')),
     )
     bug_url = 'https://github.com/git-cola/git-cola/issues'
-    bug_link = link(bug_url, bug_url)
+    bug_link = qtutils.link(bug_url, bug_url)
     scope = dict(bug_link=bug_link)
     prelude = (
         N_(
@@ -447,7 +429,7 @@ def translators_text():
     )
 
     bug_url = 'https://github.com/git-cola/git-cola/issues'
-    bug_link = link(bug_url, bug_url)
+    bug_link = qtutils.link(bug_url, bug_url)
     scope = dict(bug_link=bug_link)
 
     prelude = (
@@ -487,7 +469,7 @@ def show_shortcuts():
     try:
         from qtpy import QtWebEngineWidgets  # pylint: disable=all
     except (ImportError, qtpy.PythonQtError):
-        # redhat disabled QtWebKit in their qt build but don't punish the users
+        # Redhat disabled QtWebKit in their Qt build but don't punish the users
         webbrowser.open_new_tab('file://' + hotkeys_html)
         return
 
