@@ -1,5 +1,4 @@
 """Provides widgets related to branches"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 
 from qtpy import QtWidgets
@@ -18,7 +17,8 @@ from .. import gitcmds
 from .. import hotkeys
 from .. import icons
 from .. import qtutils
-from .text import LineEdit
+from . import log
+from . import text
 
 
 def defer_func(parent, title, func, *args, **kwargs):
@@ -56,6 +56,8 @@ class AsyncGitActionTask(qtutils.Task):
 
 
 class BranchesWidget(QtWidgets.QFrame):
+    """A widget for displaying and performing operations on branches"""
+
     def __init__(self, context, parent):
         QtWidgets.QFrame.__init__(self, parent)
         self.model = model = context.model
@@ -117,6 +119,8 @@ class BranchesWidget(QtWidgets.QFrame):
 
 # pylint: disable=too-many-ancestors
 class BranchesTreeWidget(standard.TreeWidget):
+    """A tree widget for displaying branches"""
+
     updated = Signal()
 
     def __init__(self, context, parent=None):
@@ -194,7 +198,7 @@ class BranchesTreeWidget(standard.TreeWidget):
             self._visible = True
             if self._needs_refresh:
                 self.refresh()
-        return super(BranchesTreeWidget, self).showEvent(event)
+        return super().showEvent(event)
 
     def _toggle_expanded(self, index):
         """Toggle expanded/collapsed state when items are clicked"""
@@ -216,6 +220,13 @@ class BranchesTreeWidget(standard.TreeWidget):
         full_name = selected.refname
         menu = qtutils.create_menu(N_('Actions'), self)
 
+        visualize_action = qtutils.add_action(
+            menu, N_('Visualize'), self.visualize_branch_action
+        )
+        visualize_action.setIcon(icons.visualize())
+        menu.addAction(visualize_action)
+        menu.addSeparator()
+
         # all branches except current the current branch
         if full_name != self.current_branch:
             menu.addAction(
@@ -231,7 +242,6 @@ class BranchesTreeWidget(standard.TreeWidget):
                 menu, N_('Merge into current branch'), self.merge_action
             )
             merge_menu_action.setIcon(icons.merge())
-
             menu.addAction(merge_menu_action)
 
         # local and remote branch
@@ -276,7 +286,7 @@ class BranchesTreeWidget(standard.TreeWidget):
                 menu.addSeparator()
                 menu.addAction(delete_menu_action)
 
-        # manage upstreams for local branches
+        # manage upstream branches for local branches
         if root.name == N_('Local'):
             upstream_menu = menu.addMenu(N_('Set Upstream Branch'))
             upstream_menu.setIcon(icons.branch())
@@ -404,41 +414,65 @@ class BranchesTreeWidget(standard.TreeWidget):
         if current_branch and tracked_branch and item is not None:
             status_str = ''
             if ahead > 0:
-                status_str += '%s%s' % (uchr(0x2191), ahead)
+                status_str += f'{uchr(0x2191)}{ahead}'
 
             if behind > 0:
-                status_str += '  %s%s' % (uchr(0x2193), behind)
+                status_str += f'  {uchr(0x2193)}{behind}'
 
             if status_str:
-                item.setText(0, '%s\t%s' % (item.text(0), status_str))
+                item.setText(0, f'{item.text(0)}\t{status_str}')
 
-    def git_action_async(self, action, args, kwarg=None, update_refs=False):
+    def git_action_async(
+        self, action, args, kwarg=None, update_refs=False, remote_messages=False
+    ):
+        """Execute a git action in a background task"""
         if kwarg is None:
             kwarg = {}
         task = AsyncGitActionTask(self.git_helper, action, args, kwarg, update_refs)
         progress = standard.progress(
             N_('Executing action %s') % action, N_('Updating'), self
         )
-        self.runtask.start(task, progress=progress, finish=self.git_action_completed)
+        if remote_messages:
+            result_handler = log.show_remote_messages(self.context, self)
+        else:
+            result_handler = None
+
+        self.runtask.start(
+            task,
+            progress=progress,
+            finish=self.git_action_completed,
+            result=result_handler,
+        )
 
     def git_action_completed(self, task):
+        """Update the with the results of an async git action"""
         status, out, err = task.result
         self.git_helper.show_result(task.action, status, out, err)
         if task.update_refs:
             self.context.model.update_refs()
 
     def push_action(self):
+        """Push the selected branch to its upstream remote"""
         context = self.context
         branch = self.selected_refname()
         remote_branch = gitcmds.tracked_branch(context, branch)
+        context.settings.load()
+        push_settings = context.settings.get_gui_state_by_name('push')
+        remote_messages = push_settings.get('remote_messages', False)
         if remote_branch:
             remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
             if remote and branch_name:
                 # we assume that user wants to "Push" the selected local
                 # branch to a remote with same name
-                self.git_action_async('push', [remote, branch_name], update_refs=True)
+                self.git_action_async(
+                    'push',
+                    [remote, branch_name],
+                    update_refs=True,
+                    remote_messages=remote_messages,
+                )
 
     def rename_action(self):
+        """Rename the selected branch"""
         branch = self.selected_refname()
         new_branch, ok = qtutils.prompt(
             N_('Enter New Branch Name'), title=N_('Rename branch'), text=branch
@@ -447,17 +481,27 @@ class BranchesTreeWidget(standard.TreeWidget):
             self.git_action_async('rename', [branch, new_branch], update_refs=True)
 
     def pull_action(self):
+        """Pull the selected branch into the current branch"""
         context = self.context
         branch = self.selected_refname()
         if not branch:
             return
         remote_branch = gitcmds.tracked_branch(context, branch)
+        context.settings.load()
+        pull_settings = context.settings.get_gui_state_by_name('pull')
+        remote_messages = pull_settings.get('remote_messages', False)
         if remote_branch:
             remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
             if remote and branch_name:
-                self.git_action_async('pull', [remote, branch_name], update_refs=True)
+                self.git_action_async(
+                    'pull',
+                    [remote, branch_name],
+                    update_refs=True,
+                    remote_messages=remote_messages,
+                )
 
     def delete_action(self):
+        """Delete the selected branch"""
         branch = self.selected_refname()
         if not branch or branch == self.current_branch:
             return
@@ -477,16 +521,19 @@ class BranchesTreeWidget(standard.TreeWidget):
             cmds.do(cmds.DeleteBranch, self.context, branch)
 
     def merge_action(self):
+        """Merge the selected branch into the current branch"""
         branch = self.selected_refname()
         if branch and branch != self.current_branch:
             self.git_action_async('merge', [branch])
 
     def checkout_action(self):
+        """Checkout the selected branch"""
         branch = self.selected_refname()
         if branch and branch != self.current_branch:
             self.git_action_async('checkout', [branch], update_refs=True)
 
     def checkout_new_branch_action(self):
+        """Checkout a new branch"""
         branch = self.selected_refname()
         if branch and branch != self.current_branch:
             _, new_branch = gitcmds.parse_remote_branch(branch)
@@ -494,6 +541,12 @@ class BranchesTreeWidget(standard.TreeWidget):
                 'checkout', ['-b', new_branch, branch], update_refs=True
             )
 
+    def visualize_branch_action(self):
+        """Visualize the selected branch"""
+        branch = self.selected_refname()
+        if branch:
+            cmds.do(cmds.VisualizeRevision, self.context, branch)
+
     def selected_refname(self):
         return getattr(self.selected_item(), 'refname', None)
 
@@ -502,7 +555,7 @@ class BranchDetailsTask(qtutils.Task):
     """Lookup branch details in a background task"""
 
     def __init__(self, context, current_branch, git_helper):
-        super(BranchDetailsTask, self).__init__()
+        super().__init__()
         self.context = context
         self.current_branch = current_branch
         self.git_helper = git_helper
@@ -541,7 +594,7 @@ class BranchTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         self.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
 
 
-class TreeEntry(object):
+class TreeEntry:
     """Tree representation for the branches widget
 
     The branch widget UI displays the basename.  For intermediate names, e.g.
@@ -587,7 +640,7 @@ def create_tree_entries(names):
 
     """
     # Phase 1: build a nested dictionary representing the intermediate
-    # names in the branches.  e.g. {'xxx': {'abc': {}, 'def': {}}}
+    # names in the branches, e.g. {'xxx': {'abc': {}, 'def': {}}}
     tree_names = create_name_dict(names)
 
     # Loop over the names again, this time we'll create tree entries
@@ -626,7 +679,7 @@ def create_tree_entries(names):
 
 def create_name_dict(names):
     # Phase 1: build a nested dictionary representing the intermediate
-    # names in the branches.  e.g. {'xxx': {'abc': {}, 'def': {}}}
+    # names in the branches, e.g. {'xxx': {'abc': {}, 'def': {}}}
     tree_names = {}
     for item in names:
         part_names = tree_names
@@ -698,7 +751,7 @@ def get_toplevel_item(item):
     return parents[-1]
 
 
-class BranchesTreeHelper(object):
+class BranchesTreeHelper:
     """Save and restore the tree state"""
 
     def __init__(self, tree):
@@ -746,7 +799,7 @@ class BranchesTreeHelper(object):
         return result
 
 
-class GitHelper(object):
+class GitHelper:
     def __init__(self, context):
         self.context = context
         self.git = context.git
@@ -782,7 +835,7 @@ class BranchesFilterWidget(QtWidgets.QWidget):
         self.tree = tree
 
         hint = N_('Filter branches...')
-        self.text = LineEdit(parent=self, clear_button=True)
+        self.text = text.LineEdit(parent=self, clear_button=True)
         self.text.setToolTip(hint)
         self.setFocusProxy(self.text)
         self._filter = None
@@ -790,26 +843,25 @@ class BranchesFilterWidget(QtWidgets.QWidget):
         self.main_layout = qtutils.hbox(defs.no_margin, defs.spacing, self.text)
         self.setLayout(self.main_layout)
 
-        text = self.text
         # pylint: disable=no-member
-        text.textChanged.connect(self.apply_filter)
+        self.text.textChanged.connect(self.apply_filter)
         self.tree.updated.connect(self.apply_filter, type=Qt.QueuedConnection)
 
     def apply_filter(self):
-        text = get(self.text)
-        if text == self._filter:
+        value = get(self.text)
+        if value == self._filter:
             return
         self._apply_bold(self._filter, False)
-        self._filter = text
-        if text:
-            self._apply_bold(text, True)
+        self._filter = value
+        if value:
+            self._apply_bold(value, True)
 
-    def _apply_bold(self, text, value):
+    def _apply_bold(self, value, is_bold):
         match = Qt.MatchContains | Qt.MatchRecursive
-        children = self.tree.findItems(text, match)
+        children = self.tree.findItems(value, match)
 
         for child in children:
             if child.childCount() == 0:
                 font = child.font(0)
-                font.setBold(value)
+                font.setBold(is_bold)
                 child.setFont(0, font)
