@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 
 from qtpy import QtCore
@@ -24,12 +23,14 @@ from ..models import prefs
 from ..qtutils import get
 from ..utils import Group
 from . import defs
+from . import standard
 from .selectcommits import select_commits
 from .spellcheck import SpellCheckLineEdit, SpellCheckTextEdit
 from .text import anchor_mode
 
 
 class CommitMessageEditor(QtWidgets.QFrame):
+    commit_finished = Signal(object)
     cursor_changed = Signal(int, int)
     down = Signal()
     up = Signal()
@@ -67,7 +68,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.move_up = actions.move_up(self)
         self.move_down = actions.move_down(self)
 
-        # Menu acctions
+        # Menu actions
         self.menu_actions = menu_actions = [
             self.signoff_action,
             self.commit_action,
@@ -95,6 +96,11 @@ class CommitMessageEditor(QtWidgets.QFrame):
             text=N_('Commit@@verb'), tooltip=commit_button_tooltip, icon=icons.commit()
         )
         self.commit_group = Group(self.commit_action, self.commit_button)
+        self.commit_progress_bar = standard.progress_bar(
+            self,
+            disable=(self.commit_button, self.summary, self.description),
+        )
+        self.commit_progress_bar.setMaximumHeight(defs.small_icon)
 
         self.actions_menu = qtutils.create_menu(N_('Actions'), self)
         self.actions_button = qtutils.create_toolbutton(
@@ -174,7 +180,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
             self.check_spelling_action, self.toggle_check_spelling
         )
 
-        # Handle the one-off autowrapping
+        # Handle the one-off auto-wrapping
         qtutils.connect_action_bool(self.autowrap_action, self.set_linebreak)
 
         self.summary.accepted.connect(self.focus_description)
@@ -183,6 +189,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.model.commit_message_changed.connect(
             self.set_commit_message, type=Qt.QueuedConnection
         )
+        self.commit_finished.connect(self._commit_finished, type=Qt.QueuedConnection)
 
         self.summary.cursor_changed.connect(self.cursor_changed.emit)
         self.description.cursor_changed.connect(
@@ -248,7 +255,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.focus_description()
 
     def commit_message(self, raw=True):
-        """Return the commit message as a unicode string"""
+        """Return the commit message as a Unicode string"""
         summary = get(self.summary)
         if raw:
             description = get(self.description)
@@ -457,7 +464,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
                 N_(
                     'This commit has already been published.\n'
                     'This operation will rewrite published history.\n'
-                    'You probably don\'t want to do this.'
+                    "You probably don't want to do this."
                 ),
                 N_('Amend the published commit?'),
                 N_('Amend Commit'),
@@ -468,9 +475,24 @@ class CommitMessageEditor(QtWidgets.QFrame):
             return
         no_verify = get(self.bypass_commit_hooks_action)
         sign = get(self.sign_action)
-        cmds.do(cmds.Commit, context, amend, msg, sign, no_verify=no_verify)
         self.bypass_commit_hooks_action.setChecked(False)
 
+        task = qtutils.SimpleTask(
+            cmds.run(cmds.Commit, context, amend, msg, sign, no_verify=no_verify)
+        )
+        self.context.runtask.start(
+            task,
+            finish=self.commit_finished.emit,
+            progress=self.commit_progress_bar,
+        )
+
+    def _commit_finished(self, task):
+        """Reset widget state on completion of the commit task"""
+        title = N_('Commit failed')
+        status, out, err = task.result
+        Interaction.command(title, 'git commit', status, out, err)
+        self.setFocus()
+
     def build_fixup_menu(self):
         self.build_commits_menu(
             cmds.LoadFixupMessage,
@@ -662,5 +684,5 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
 
     def setFont(self, font):
         SpellCheckTextEdit.setFont(self, font)
-        metrics = self.fontMetrics()
-        self.setMinimumSize(QtCore.QSize(metrics.width('MMMM'), metrics.height() * 2))
+        width, height = qtutils.text_size(font, 'MMMM')
+        self.setMinimumSize(QtCore.QSize(width, height * 2))
