@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 import re
 import time
 
@@ -18,7 +17,7 @@ from . import defs
 from .text import HintedLineEdit
 
 
-class ValidateRegex(object):
+class ValidateRegex:
     def __init__(self, regex):
         self.regex = re.compile(regex)  # regex to scrub
 
@@ -35,7 +34,7 @@ class RemoteValidator(QtGui.QValidator):
     """Prevent invalid remote names"""
 
     def __init__(self, parent=None):
-        super(RemoteValidator, self).__init__(parent)
+        super().__init__(parent)
         self._validate = ValidateRegex(r'[ \t\\/]')
 
     def validate(self, string, idx):
@@ -46,7 +45,7 @@ class BranchValidator(QtGui.QValidator):
     """Prevent invalid branch names"""
 
     def __init__(self, git, parent=None):
-        super(BranchValidator, self).__init__(parent)
+        super().__init__(parent)
         self._git = git
         self._validate = ValidateRegex(r'[ \t\\]')  # forward-slash is okay
 
@@ -205,20 +204,27 @@ class CompletionLineEdit(HintedLineEdit):
         self.complete()
 
     def close_popup(self):
-        if self.popup().isVisible():
-            self.popup().close()
+        """Close the completion popup"""
+        self.popup().close()
 
     def _completions_updated(self):
+        """Select the first completion item when completions are updated"""
         popup = self.popup()
+        if self._completion_model.rowCount() == 0:
+            popup.hide()
+            return
         if not popup.isVisible():
             if not self.hasFocus() or not self._show_all_completions:
                 return
-            popup.show()
-        # Select the first item
+        self.select_first_completion()
+
+    def select_first_completion(self):
+        """Select the first item in the completion model"""
         idx = self._completion_model.index(0, 0)
-        selection = QtCore.QItemSelection(idx, idx)
-        mode = QtCore.QItemSelectionModel.Select
-        popup.selectionModel().select(selection, mode)
+        mode = (
+            QtCore.QItemSelectionModel.Rows | QtCore.QItemSelectionModel.SelectCurrent
+        )
+        self.popup().selectionModel().setCurrentIndex(idx, mode)
 
     def selected_completion(self):
         """Return the selected completion item"""
@@ -246,6 +252,17 @@ class CompletionLineEdit(HintedLineEdit):
                 result = True
         return result
 
+    def show_popup(self):
+        """Display the completion popup"""
+        self.refresh()
+        x_val = self.x()
+        y_val = self.y() + self.height()
+        point = QtCore.QPoint(x_val, y_val)
+        mapped = self.parent().mapToGlobal(point)
+        popup = self.popup()
+        popup.move(mapped.x(), mapped.y())
+        popup.show()
+
     # Qt overrides
     def event(self, event):
         """Override QWidget::event() for tab completion"""
@@ -259,22 +276,26 @@ class CompletionLineEdit(HintedLineEdit):
             return True
 
         # Make sure the popup goes away during teardown
-        if event_type == QtCore.QEvent.Hide:
+        if event_type == QtCore.QEvent.Close:
             self.close_popup()
+        elif event_type == QtCore.QEvent.Hide:
+            self.popup().hide()
 
-        return super(CompletionLineEdit, self).event(event)
+        return super().event(event)
 
     def keyPressEvent(self, event):
         """Process completion and navigation events"""
-        super(CompletionLineEdit, self).keyPressEvent(event)
-        visible = self.popup().isVisible()
+        super().keyPressEvent(event)
 
-        # Hide the popup when the field is empty
+        popup = self.popup()
+        visible = popup.isVisible()
+
+        # Hide the popup when the field becomes empty.
         is_empty = not self.value()
-        if is_empty:
+        if is_empty and event.modifiers() != Qt.ControlModifier:
             self.cleared.emit()
             if visible:
-                self.popup().hide()
+                popup.hide()
 
         # Activation keys select the completion when pressed and emit the
         # activated signal.  Navigation keys have lower priority, and only
@@ -289,6 +310,15 @@ class CompletionLineEdit(HintedLineEdit):
         if navigation:
             signal = getattr(self, navigation)
             signal.emit()
+            return
+
+        # Show the popup when Ctrl-Space is pressed.
+        if (
+            not visible
+            and key == Qt.Key_Space
+            and event.modifiers() == Qt.ControlModifier
+        ):
+            self.show_popup()
 
 
 class GatherCompletionsThread(QtCore.QThread):
@@ -695,6 +725,43 @@ class GitLogCompletionModel(GitRefCompletionModel):
             self._start_gathering_paths, type=Qt.QueuedConnection
         )
 
+    def matches(self):
+        """Return candidate values for completion"""
+        matches = super().matches()
+        return [
+            '--all',
+            '--all-match',
+            '--author',
+            '--after=two.days.ago',
+            '--basic-regexp',
+            '--before=two.days.ago',
+            '--branches',
+            '--committer',
+            '--exclude',
+            '--extended-regexp',
+            '--find-object',
+            '--first-parent',
+            '--fixed-strings',
+            '--full-diff',
+            '--grep',
+            '--invert-grep',
+            '--merges',
+            '--no-merges',
+            '--not',
+            '--perl-regexp',
+            '--pickaxe-all',
+            '--pickaxe-regex',
+            '--regexp-ignore-case',
+            '--tags',
+            '-D',
+            '-E',
+            '-F',
+            '-G',
+            '-P',
+            '-S',
+            '@{upstream}',
+        ] + matches
+
     def _start_gathering_paths(self):
         """Gather paths when the model changes"""
         # Debounce updates that land within 1 second of each other.
@@ -819,18 +886,7 @@ class GitDialog(QtWidgets.QDialog):
             dlg.set_text(default)
 
         dlg.show()
-
-        def show_popup():
-            x_val = dlg.lineedit.x()
-            y_val = dlg.lineedit.y() + dlg.lineedit.height()
-            point = QtCore.QPoint(x_val, y_val)
-            mapped = dlg.mapToGlobal(point)
-            dlg.lineedit.popup().move(mapped.x(), mapped.y())
-            dlg.lineedit.popup().show()
-            dlg.lineedit.refresh()
-            dlg.lineedit.setFocus()
-
-        QtCore.QTimer().singleShot(100, show_popup)
+        QtCore.QTimer().singleShot(250, dlg.lineedit.show_popup)
 
         if dlg.exec_() == cls.Accepted:
             return dlg.text()
