@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 import collections
 import itertools
 import math
@@ -57,7 +56,7 @@ def git_dag(context, args=None, existing_view=None, show=True):
     return view
 
 
-class FocusRedirectProxy(object):
+class FocusRedirectProxy:
     """Redirect actions from the main widget to child widgets"""
 
     def __init__(self, *widgets):
@@ -79,7 +78,7 @@ class FocusRedirectProxy(object):
         return func(*args, **kwargs)
 
 
-class ViewerMixin(object):
+class ViewerMixin:
     """Implementations must provide selected_items()"""
 
     def __init__(self):
@@ -105,7 +104,7 @@ class ViewerMixin(object):
         return result
 
     def selected_oids(self):
-        """Return the currently selected comit object IDs"""
+        """Return the currently selected commit object IDs"""
         return [i.commit for i in self.selected_items()]
 
     def clicked_oid(self):
@@ -200,7 +199,7 @@ class ViewerMixin(object):
         """Show a full directory diff for the selected commit"""
         context = self.context
         self.with_oid(
-            lambda oid: cmds.difftool_launch(
+            lambda oid: difftool.difftool_launch(
                 context, left=oid, left_take_magic=True, dir_diff=True
             )
         )
@@ -350,7 +349,7 @@ def set_icon(icon, action):
 
 
 def viewer_actions(widget):
-    """Return commont actions across the tree and graph widgets"""
+    """Return common actions across the tree and graph widgets"""
     return {
         'diff_this_selected': set_icon(
             icons.compare(),
@@ -479,6 +478,73 @@ def viewer_actions(widget):
     }
 
 
+class GitDagLineEdit(completion.GitLogLineEdit):
+    """The text input field for specifying "git log" options"""
+
+    def __init__(self, context):
+        super().__init__(context)
+        self._action_filter_to_current_author = qtutils.add_action(
+            self, N_('Commits authored by me'), self._filter_to_current_author
+        )
+        self._action_pickaxe_search = qtutils.add_action(
+            self, N_('Pickaxe search for changes containing text'), self._pickaxe_search
+        )
+        self._action_grep_search = qtutils.add_action(
+            self,
+            N_('Search commit messages'),
+            self._grep_search,
+        )
+        self._action_no_merges = qtutils.add_action(
+            self, N_('Ignore merge commits'), self._no_merges
+        )
+
+    def contextMenuEvent(self, event):
+        """Adds custom actions to the default context menu"""
+        event_pos = event.pos()
+        menu = self.createStandardContextMenu()
+        menu.addSeparator()
+        actions = menu.actions()
+        first_action = actions[0]
+        menu.insertAction(first_action, self._action_pickaxe_search)
+        menu.insertAction(first_action, self._action_filter_to_current_author)
+        menu.insertAction(first_action, self._action_grep_search)
+        menu.insertAction(first_action, self._action_no_merges)
+        menu.insertSeparator(first_action)
+        menu.exec_(self.mapToGlobal(event_pos))
+
+    def insert(self, text):
+        """Insert text at the beginning of the current text"""
+        value = self.value()
+        if value:
+            text = f'{text} {value}'
+        self.setText(text)
+        self.close_popup()
+
+    def _filter_to_current_author(self):
+        """Filter to commits by the current author/user"""
+        _, email = self.context.cfg.get_author()
+        author_filter = '--author=' + email
+        self.insert(author_filter)
+
+    def _pickaxe_search(self):
+        """Pickaxe search for changes containing text"""
+        self.insert('-G"search"')
+        start = len('-G"')
+        length = len('search')
+        self.setSelection(start, length)
+
+    def _grep_search(self):
+        """Pickaxe search for changes containing text"""
+        self.insert('--grep="search"')
+        start = len('--grep="')
+        length = len('search')
+        self.setSelection(start, length)
+
+    def _no_merges(self):
+        """Ignore merge commits"""
+        self.insert('--no-merges')
+
+
 class CommitTreeWidgetItem(QtWidgets.QTreeWidgetItem):
     """Custom TreeWidgetItem used in to build the commit tree widget"""
 
@@ -530,7 +596,7 @@ class CommitTreeWidget(standard.TreeWidget, ViewerMixin):
     def export_state(self):
         """Export the widget's state"""
         # The base class method is intentionally overridden because we only
-        # care about the details below for this subwidget.
+        # care about the details below for this sub-widget.
         state = {}
         state['column_widths'] = self.column_widths()
         return state
@@ -680,7 +746,7 @@ class GitDAG(standard.MainWindow):
     commits_selected = Signal(object)
 
     def __init__(self, context, params, parent=None):
-        super(GitDAG, self).__init__(parent)
+        super().__init__(parent)
 
         self.setMinimumSize(420, 420)
 
@@ -699,7 +765,7 @@ class GitDAG(standard.MainWindow):
         self.force_refresh = False
 
         self.thread = None
-        self.revtext = completion.GitLogLineEdit(context)
+        self.revtext = GitDagLineEdit(context)
         self.maxresults = standard.SpinBox()
 
         self.zoom_out = qtutils.create_action_button(
@@ -931,7 +997,6 @@ class GitDAG(standard.MainWindow):
         # self.force_refresh triggers an Unconditional redraw
         self.force_refresh = True
         cmds.do(cmds.Refresh, self.context)
-        self.force_refresh = False
 
     def display(self):
         """Update the view when the Git refs change"""
@@ -969,6 +1034,7 @@ class GitDAG(standard.MainWindow):
         self.old_oids = oids
         self.old_count = count
         self.old_refs = refs
+        self.force_refresh = False
 
     def select_commits(self, commits):
         self.selection = commits
@@ -1020,7 +1086,7 @@ class GitDAG(standard.MainWindow):
     def diff_commits(self, left, right):
         paths = self.params.paths()
         if paths:
-            cmds.difftool_launch(self.context, left=left, right=right, paths=paths)
+            difftool.difftool_launch(self.context, left=left, right=right, paths=paths)
         else:
             difftool.diff_commits(self.context, self, left, right)
 
@@ -1041,12 +1107,12 @@ class GitDAG(standard.MainWindow):
         bottom, top = self.treewidget.selected_commit_range()
         if not top:
             return
-        cmds.difftool_launch(
+        difftool.difftool_launch(
             self.context, left=bottom, left_take_parent=True, right=top, paths=files
         )
 
     def grab_file(self, filename):
-        """Save the selected file from the filelist widget"""
+        """Save the selected file from the file list widget"""
         oid = self.treewidget.selected_oid()
         model = browse.BrowseModel(oid, filename=filename)
         browse.save_path(self.context, filename, model)
@@ -1112,7 +1178,7 @@ class ReaderThread(QtCore.QThread):
         self.wait()
 
 
-class Cache(object):
+class Cache:
     _label_font = None
 
     @classmethod
@@ -1195,12 +1261,12 @@ class Edge(QtWidgets.QGraphicsItem):
             path.moveTo(self.source.x(), self.source.y())
             path.lineTo(self.dest.x(), self.dest.y())
         else:
-            # Define points starting from source
+            # Define points starting from the source.
             point1 = QPointF(self.source.x(), self.source.y())
             point2 = QPointF(point1.x(), point1.y() - connector_length)
             point3 = QPointF(point2.x() + arc_rect, point2.y() - arc_rect)
 
-            # Define points starting from dest
+            # Define points starting from the destination.
             point4 = QPointF(self.dest.x(), self.dest.y())
             point5 = QPointF(point4.x(), point3.y() - arc_rect)
             point6 = QPointF(point5.x() - arc_rect, point5.y() + arc_rect)
@@ -1210,8 +1276,8 @@ class Edge(QtWidgets.QGraphicsItem):
             start_angle_arc2 = 90
             span_angle_arc2 = -90
 
-            # If the dest is at the left of the source, then we
-            # need to reverse some values
+            # If the destination is at the left of the source, then we need to
+            # reverse some values.
             if self.source.x() > self.dest.x():
                 point3 = QPointF(point2.x() - arc_rect, point3.y())
                 point6 = QPointF(point5.x() + arc_rect, point6.y())
@@ -1235,7 +1301,7 @@ class Edge(QtWidgets.QGraphicsItem):
         painter.drawPath(self.path)
 
 
-class EdgeColor(object):
+class EdgeColor:
     """An edge color factory"""
 
     current_color_index = 0
@@ -1252,27 +1318,23 @@ class EdgeColor(object):
     def update_colors(cls, theme):
         """Update the colors based on the color theme"""
         if theme.is_dark or theme.is_palette_dark:
-            cls.colors.extend(
-                [
-                    QtGui.QColor(Qt.red).lighter(),
-                    QtGui.QColor(Qt.cyan).lighter(),
-                    QtGui.QColor(Qt.magenta).lighter(),
-                    QtGui.QColor(Qt.green).lighter(),
-                    QtGui.QColor(Qt.yellow).lighter(),
-                ]
-            )
+            cls.colors.extend([
+                QtGui.QColor(Qt.red).lighter(),
+                QtGui.QColor(Qt.cyan).lighter(),
+                QtGui.QColor(Qt.magenta).lighter(),
+                QtGui.QColor(Qt.green).lighter(),
+                QtGui.QColor(Qt.yellow).lighter(),
+            ])
         else:
-            cls.colors.extend(
-                [
-                    QtGui.QColor(Qt.blue),
-                    QtGui.QColor(Qt.darkRed),
-                    QtGui.QColor(Qt.darkCyan),
-                    QtGui.QColor(Qt.darkMagenta),
-                    QtGui.QColor(Qt.darkGreen),
-                    QtGui.QColor(Qt.darkYellow),
-                    QtGui.QColor(Qt.darkBlue),
-                ]
-            )
+            cls.colors.extend([
+                QtGui.QColor(Qt.blue),
+                QtGui.QColor(Qt.darkRed),
+                QtGui.QColor(Qt.darkCyan),
+                QtGui.QColor(Qt.darkMagenta),
+                QtGui.QColor(Qt.darkGreen),
+                QtGui.QColor(Qt.darkYellow),
+                QtGui.QColor(Qt.darkBlue),
+            ])
 
     @classmethod
     def cycle(cls):
@@ -1386,7 +1448,7 @@ class Commit(QtWidgets.QGraphicsItem):
         return self.item_shape
 
     def paint(self, painter, option, _widget):
-        # Do not draw outside the exposed rect
+        # Do not draw outside the exposed rectangle.
         painter.setClipRect(option.exposedRect)
 
         # Draw ellipse
@@ -1751,7 +1813,9 @@ class GraphView(QtWidgets.QGraphicsView, ViewerMixin):
             commit = self.commits[-1]
             items.append(self.items[commit.oid])
 
-        self.setSceneRect(self.scene().itemsBoundingRect())
+        bounds = self.scene().itemsBoundingRect()
+        bounds.adjust(-64, 0, 0, 0)
+        self.setSceneRect(bounds)
         self.fit_view_to_items(items)
 
     def zoom_to_fit(self):
@@ -2016,7 +2080,7 @@ class GraphView(QtWidgets.QGraphicsView, ViewerMixin):
     def reset_columns(self):
         # Some children of displayed commits might not be accounted in
         # 'commits' list. It is common case during loading of big graph.
-        # But, they are assigned a column that must be reseted. Hence, use
+        # But, they are assigned a column that must be reset. Hence, use
         # depth-first traversal to reset all columns assigned.
         for node in self.commits:
             if node.column is None:
