@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from qtpy import QtCore
 from qtpy import QtWidgets
 from qtpy.QtCore import Qt
@@ -302,13 +300,13 @@ class Grep(Dialog):
 
     def export_state(self):
         """Export persistent settings"""
-        state = super(Grep, self).export_state()
+        state = super().export_state()
         state['sizes'] = get(self.splitter)
         return state
 
     def apply_state(self, state):
         """Apply persistent settings"""
-        result = super(Grep, self).apply_state(state)
+        result = super().apply_state(state)
         try:
             self.splitter.setSizes(state['sizes'])
         except (AttributeError, KeyError, ValueError, TypeError):
@@ -344,7 +342,7 @@ class PreviewTask(qtutils.Task):
     def task(self):
         try:
             self.content = core.read(self.filename, errors='ignore')
-        except IOError:
+        except OSError:
             pass
         return (self.filename, self.content, self.line_number)
 
@@ -360,7 +358,7 @@ class PreviewTextView(VimTextBrowser):
         self.runtask = qtutils.RunTask(parent=self)
 
     def preview(self, filename, line_number):
-        """Preview the a file at the specified line number"""
+        """Preview a file at the specified line number"""
 
         if filename != self.filename:
             request = PreviewTask(filename, line_number)
@@ -371,7 +369,7 @@ class PreviewTextView(VimTextBrowser):
     def clear(self):
         self.filename = ''
         self.content = ''
-        super(PreviewTextView, self).clear()
+        super().clear()
 
     def show_preview(self, task):
         """Show the results of the asynchronous file read"""
