@@ -1,5 +1,4 @@
 """Main UI for authoring commits and other Git Cola interactions"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import os
 from functools import partial
 
@@ -48,7 +47,6 @@ from . import editremotes
 from . import grep
 from . import log
 from . import merge
-from . import patch
 from . import prefs as prefs_widget
 from . import recent
 from . import remote
@@ -154,14 +152,15 @@ class MainView(standard.MainWindow):
         self.position_label.setFont(font)
 
         # make the position label fixed size to avoid layout issues
-        metrics = self.position_label.fontMetrics()
-        width = metrics.width('99:999') + defs.spacing
+        text_width = qtutils.text_width(font, '99:999')
+        width = text_width + defs.spacing
         self.position_label.setMinimumWidth(width)
 
         editor = commitmsg.CommitMessageEditor(context, self)
         self.commiteditor = editor
         self.commitdock = create_dock('Commit', N_('Commit'), self, widget=editor)
         titlebar = self.commitdock.titleBarWidget()
+        titlebar.add_title_widget(self.commiteditor.commit_progress_bar)
         titlebar.add_corner_widget(self.position_label)
 
         # "Console" widget
@@ -180,9 +179,10 @@ class MainView(standard.MainWindow):
         )
         self.diffviewer = self.diffdock.widget()
         self.diffviewer.set_diff_type(self.model.diff_type)
-
+        self.diffviewer.enable_filename_tracking()
         self.diffeditor = self.diffviewer.text
         titlebar = self.diffdock.titleBarWidget()
+        titlebar.add_title_widget(self.diffviewer.filename)
         titlebar.add_corner_widget(self.diffviewer.options)
 
         # All Actions
@@ -210,7 +210,7 @@ class MainView(standard.MainWindow):
         self.undo_commit_action.setIcon(icons.style_dialog_discard())
 
         self.unstage_selected_action = add_action(
-            self, N_('Unstage From Commit'), cmds.run(cmds.UnstageSelected, context)
+            self, N_('Unstage'), cmds.run(cmds.UnstageSelected, context)
         )
         self.unstage_selected_action.setIcon(icons.remove())
 
@@ -221,7 +221,7 @@ class MainView(standard.MainWindow):
 
         self.stage_modified_action = add_action(
             self,
-            N_('Stage Changed Files To Commit'),
+            cmds.StageModified.name(),
             cmds.run(cmds.StageModified, context),
             hotkeys.STAGE_MODIFIED,
         )
@@ -229,42 +229,38 @@ class MainView(standard.MainWindow):
 
         self.stage_untracked_action = add_action(
             self,
-            N_('Stage All Untracked'),
+            cmds.StageUntracked.name(),
             cmds.run(cmds.StageUntracked, context),
             hotkeys.STAGE_UNTRACKED,
         )
         self.stage_untracked_action.setIcon(icons.add())
 
         self.apply_patches_action = add_action(
-            self, N_('Apply Patches...'), partial(patch.apply_patches, context)
+            self, N_('Apply Patches...'), partial(diff.apply_patches, context)
         )
         self.apply_patches_action.setIcon(icons.diff())
 
-        self.apply_patches_abort_action = add_action(
+        self.apply_patches_abort_action = qtutils.add_action_with_tooltip(
             self,
             N_('Abort Applying Patches...'),
+            N_('Abort the current "git am" patch session'),
             cmds.run(cmds.AbortApplyPatch, context),
         )
         self.apply_patches_abort_action.setIcon(icons.style_dialog_discard())
-        self.apply_patches_abort_action.setToolTip(
-            N_('Abort the current "git am" patch session')
-        )
 
-        self.apply_patches_continue_action = add_action(
+        self.apply_patches_continue_action = qtutils.add_action_with_tooltip(
             self,
             N_('Continue Applying Patches'),
+            N_('Commit the current state and continue applying patches'),
             cmds.run(cmds.ApplyPatchesContinue, context),
         )
-        self.apply_patches_continue_action.setToolTip(
-            N_('Commit the current state and continue applying patches')
-        )
         self.apply_patches_continue_action.setIcon(icons.commit())
 
-        self.apply_patches_skip_action = add_action(
-            self, N_('Skip Current Patch'), cmds.run(cmds.ApplyPatchesContinue, context)
-        )
-        self.apply_patches_skip_action.setToolTip(
-            N_('Skip applying the current patch and continue applying patches')
+        self.apply_patches_skip_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Skip Current Patch'),
+            N_('Skip applying the current patch and continue applying patches'),
+            cmds.run(cmds.ApplyPatchesContinue, context),
         )
         self.apply_patches_skip_action.setIcon(icons.discard())
 
@@ -303,7 +299,7 @@ class MainView(standard.MainWindow):
             self,
             cmds.Refresh.name(),
             cmds.run(cmds.Refresh, context),
-            *hotkeys.REFRESH_HOTKEYS
+            *hotkeys.REFRESH_HOTKEYS,
         )
         self.rescan_action.setIcon(icons.sync())
 
@@ -383,18 +379,30 @@ class MainView(standard.MainWindow):
         )
         self.add_submodule_action.setIcon(icons.add())
 
-        self.fetch_action = add_action(
-            self, N_('Fetch...'), partial(remote.fetch, context), hotkeys.FETCH
+        self.fetch_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Fetch...'),
+            N_('Fetch from one or more remotes using "git fetch"'),
+            partial(remote.fetch, context),
+            hotkeys.FETCH,
         )
         self.fetch_action.setIcon(icons.download())
 
-        self.push_action = add_action(
-            self, N_('Push...'), partial(remote.push, context), hotkeys.PUSH
+        self.push_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Push...'),
+            N_('Push to one or more remotes using "git push"'),
+            partial(remote.push, context),
+            hotkeys.PUSH,
         )
         self.push_action.setIcon(icons.push())
 
-        self.pull_action = add_action(
-            self, N_('Pull...'), partial(remote.pull, context), hotkeys.PULL
+        self.pull_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Pull...'),
+            N_('Integrate changes using "git pull"'),
+            partial(remote.pull, context),
+            hotkeys.PULL,
         )
         self.pull_action.setIcon(icons.pull())
 
@@ -410,56 +418,62 @@ class MainView(standard.MainWindow):
         )
         self.open_repo_new_action.setIcon(icons.folder())
 
-        self.stash_action = add_action(
-            self, N_('Stash...'), partial(stash.view, context), hotkeys.STASH
+        self.stash_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Stash...'),
+            N_('Temporarily stash away uncommitted changes using "git stash"'),
+            partial(stash.view, context),
+            hotkeys.STASH,
         )
         self.stash_action.setIcon(icons.commit())
 
-        self.reset_soft_action = add_action(
-            self, N_('Reset Branch (Soft)'), partial(guicmds.reset_soft, context)
+        self.reset_soft_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Reset Branch (Soft)'),
+            cmds.ResetSoft.tooltip('<commit>'),
+            partial(guicmds.reset_soft, context),
         )
         self.reset_soft_action.setIcon(icons.style_dialog_reset())
-        self.reset_soft_action.setToolTip(cmds.ResetSoft.tooltip('<commit>'))
 
-        self.reset_mixed_action = add_action(
+        self.reset_mixed_action = qtutils.add_action_with_tooltip(
             self,
             N_('Reset Branch and Stage (Mixed)'),
+            cmds.ResetMixed.tooltip('<commit>'),
             partial(guicmds.reset_mixed, context),
         )
         self.reset_mixed_action.setIcon(icons.style_dialog_reset())
-        self.reset_mixed_action.setToolTip(cmds.ResetMixed.tooltip('<commit>'))
 
-        self.reset_keep_action = add_action(
+        self.reset_keep_action = qtutils.add_action_with_tooltip(
             self,
             N_('Restore Worktree and Reset All (Keep Unstaged Changes)'),
+            cmds.ResetKeep.tooltip('<commit>'),
             partial(guicmds.reset_keep, context),
         )
         self.reset_keep_action.setIcon(icons.style_dialog_reset())
-        self.reset_keep_action.setToolTip(cmds.ResetKeep.tooltip('<commit>'))
 
-        self.reset_merge_action = add_action(
+        self.reset_merge_action = qtutils.add_action_with_tooltip(
             self,
             N_('Restore Worktree and Reset All (Merge)'),
+            cmds.ResetMerge.tooltip('<commit>'),
             partial(guicmds.reset_merge, context),
         )
         self.reset_merge_action.setIcon(icons.style_dialog_reset())
-        self.reset_merge_action.setToolTip(cmds.ResetMerge.tooltip('<commit>'))
 
-        self.reset_hard_action = add_action(
+        self.reset_hard_action = qtutils.add_action_with_tooltip(
             self,
             N_('Restore Worktree and Reset All (Hard)'),
+            cmds.ResetHard.tooltip('<commit>'),
             partial(guicmds.reset_hard, context),
         )
         self.reset_hard_action.setIcon(icons.style_dialog_reset())
-        self.reset_hard_action.setToolTip(cmds.ResetHard.tooltip('<commit>'))
 
-        self.restore_worktree_action = add_action(
-            self, N_('Restore Worktree'), partial(guicmds.restore_worktree, context)
+        self.restore_worktree_action = qtutils.add_action_with_tooltip(
+            self,
+            N_('Restore Worktree'),
+            cmds.RestoreWorktree.tooltip('<commit>'),
+            partial(guicmds.restore_worktree, context),
         )
         self.restore_worktree_action.setIcon(icons.edit())
-        self.restore_worktree_action.setToolTip(
-            cmds.RestoreWorktree.tooltip('<commit>')
-        )
 
         self.clone_repo_action = add_action(
             self, N_('Clone...'), partial(clone.clone, context)
@@ -1029,7 +1043,7 @@ class MainView(standard.MainWindow):
                 # Omit the current worktree from the "Open Recent" menu.
                 continue
             name = entry['name']
-            text = '%s %s %s' % (name, uchr(0x2192), directory)
+            text = f'{name} {uchr(0x2192)} {directory}'
             menu.addAction(text, cmds.run(cmd, context, directory))
 
     # Accessors
@@ -1176,7 +1190,7 @@ class MainView(standard.MainWindow):
         else:
             path_text = ''
 
-        title = '%s: %s %s%s' % (project, curbranch, alert_text, path_text)
+        title = f'{project}: {curbranch} {alert_text}{path_text}'
         self.setWindowTitle(title)
 
     def update_actions(self):
@@ -1328,11 +1342,11 @@ class MainView(standard.MainWindow):
             cls = 'first-warning'
         else:
             cls = 'good'
-        div = '<div class="%s">%s</div>' % (cls, display_content)
+        div = f'<div class="{cls}">{display_content}</div>'
         self.position_label.setText(css + div)
 
 
-class FocusProxy(object):
+class FocusProxy:
     """Proxy over child widgets and operate on the focused widget"""
 
     def __init__(self, *widgets):
@@ -1369,7 +1383,7 @@ class FocusProxy(object):
         return callback
 
     def delete(self):
-        """Specialized delete() to deal with QLineEdit vs QTextEdit"""
+        """Specialized delete() to deal with QLineEdit vs. QTextEdit"""
         focus = self.focus('delete')
         if hasattr(focus, 'del_'):
             focus.del_()
