@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from qtpy import QtCore
 from qtpy import QtWidgets
 
@@ -131,6 +129,33 @@ class RepoFormWidget(FormWidget):
         self.patches_directory = standard.DirectoryPathLineEdit(patches_directory, self)
         self.patches_directory.setToolTip(tooltip)
 
+        tooltip = N_(
+            """
+This option determines how the supplied commit message should be
+cleaned up before committing.
+
+The <mode> can be strip, whitespace, verbatim, scissors or default.
+
+strip
+    Strip leading and trailing empty lines, trailing whitespace,
+    commentary and collapse consecutive empty lines.
+
+whitespace
+    Same as strip except #commentary is not removed.
+
+verbatim
+    Do not change the message at all.
+
+scissors
+    Same as whitespace except that everything from (and including) the line
+    found below is truncated, if the message is to be edited.
+    "#" can be customized with core.commentChar.
+
+    # ------------------------ >8 ------------------------"""
+        )
+        self.commit_cleanup = qtutils.combo(
+            prefs.commit_cleanup_modes(), tooltip=tooltip
+        )
         self.diff_context = standard.SpinBox(value=5, mini=2, maxi=9995)
         self.merge_verbosity = standard.SpinBox(value=5, maxi=5)
         self.merge_summary = qtutils.checkbox(checked=True)
@@ -164,6 +189,7 @@ class RepoFormWidget(FormWidget):
         self.add_row(N_('User Name'), self.name)
         self.add_row(N_('Email Address'), self.email)
         self.add_row(N_('Log Date Format'), self.logdate)
+        self.add_row(N_('Commit Message Cleanup'), self.commit_cleanup)
         self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
         self.add_row(N_('Number of Diff Context Lines'), self.diff_context)
         self.add_row(N_('Patches Directory'), self.patches_directory)
@@ -182,38 +208,37 @@ class RepoFormWidget(FormWidget):
             N_('Check Published Commits when Amending'), self.check_published_commits
         )
 
-        self.set_config(
-            {
-                prefs.AUTOTEMPLATE: (self.autotemplate, Defaults.autotemplate),
-                prefs.AUTOCOMPLETE_PATHS: (
-                    self.autocomplete_paths,
-                    Defaults.autocomplete_paths,
-                ),
-                prefs.CHECK_CONFLICTS: (self.check_conflicts, Defaults.check_conflicts),
-                prefs.CHECK_PUBLISHED_COMMITS: (
-                    self.check_published_commits,
-                    Defaults.check_published_commits,
-                ),
-                prefs.DIFFCONTEXT: (self.diff_context, Defaults.diff_context),
-                prefs.DISPLAY_UNTRACKED: (
-                    self.display_untracked,
-                    Defaults.display_untracked,
-                ),
-                prefs.ENABLE_GRAVATAR: (self.enable_gravatar, Defaults.enable_gravatar),
-                prefs.LOGDATE: (self.logdate, Defaults.logdate),
-                prefs.MERGE_DIFFSTAT: (self.merge_diffstat, Defaults.merge_diffstat),
-                prefs.MERGE_SUMMARY: (self.merge_summary, Defaults.merge_summary),
-                prefs.MERGE_VERBOSITY: (self.merge_verbosity, Defaults.merge_verbosity),
-                prefs.PATCHES_DIRECTORY: (
-                    self.patches_directory,
-                    Defaults.patches_directory,
-                ),
-                prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
-                prefs.SHOW_PATH: (self.show_path, Defaults.show_path),
-                prefs.USER_NAME: (self.name, ''),
-                prefs.USER_EMAIL: (self.email, ''),
-            }
-        )
+        self.set_config({
+            prefs.AUTOTEMPLATE: (self.autotemplate, Defaults.autotemplate),
+            prefs.AUTOCOMPLETE_PATHS: (
+                self.autocomplete_paths,
+                Defaults.autocomplete_paths,
+            ),
+            prefs.CHECK_CONFLICTS: (self.check_conflicts, Defaults.check_conflicts),
+            prefs.CHECK_PUBLISHED_COMMITS: (
+                self.check_published_commits,
+                Defaults.check_published_commits,
+            ),
+            prefs.COMMIT_CLEANUP: (self.commit_cleanup, Defaults.commit_cleanup),
+            prefs.DIFFCONTEXT: (self.diff_context, Defaults.diff_context),
+            prefs.DISPLAY_UNTRACKED: (
+                self.display_untracked,
+                Defaults.display_untracked,
+            ),
+            prefs.ENABLE_GRAVATAR: (self.enable_gravatar, Defaults.enable_gravatar),
+            prefs.LOGDATE: (self.logdate, Defaults.logdate),
+            prefs.MERGE_DIFFSTAT: (self.merge_diffstat, Defaults.merge_diffstat),
+            prefs.MERGE_SUMMARY: (self.merge_summary, Defaults.merge_summary),
+            prefs.MERGE_VERBOSITY: (self.merge_verbosity, Defaults.merge_verbosity),
+            prefs.PATCHES_DIRECTORY: (
+                self.patches_directory,
+                Defaults.patches_directory,
+            ),
+            prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
+            prefs.SHOW_PATH: (self.show_path, Defaults.show_path),
+            prefs.USER_NAME: (self.name, ''),
+            prefs.USER_EMAIL: (self.email, ''),
+        })
 
 
 class SettingsFormWidget(FormWidget):
@@ -261,38 +286,36 @@ class SettingsFormWidget(FormWidget):
         self.add_row(N_('Check spelling'), self.check_spelling)
         self.add_row(N_('Ctrl+MouseWheel to Zoom'), self.mouse_zoom)
 
-        self.set_config(
-            {
-                prefs.SAVEWINDOWSETTINGS: (
-                    self.save_window_settings,
-                    Defaults.save_window_settings,
-                ),
-                prefs.TABWIDTH: (self.tabwidth, Defaults.tabwidth),
-                prefs.EXPANDTAB: (self.expandtab, Defaults.expandtab),
-                prefs.TEXTWIDTH: (self.textwidth, Defaults.textwidth),
-                prefs.LINEBREAK: (self.linebreak, Defaults.linebreak),
-                prefs.MAXRECENT: (self.maxrecent, Defaults.maxrecent),
-                prefs.SORT_BOOKMARKS: (self.sort_bookmarks, Defaults.sort_bookmarks),
-                prefs.DIFFTOOL: (self.difftool, Defaults.difftool),
-                prefs.EDITOR: (self.editor, fallback_editor()),
-                prefs.HISTORY_BROWSER: (
-                    self.historybrowser,
-                    prefs.default_history_browser(),
-                ),
-                prefs.BLAME_VIEWER: (self.blameviewer, Defaults.blame_viewer),
-                prefs.MERGE_KEEPBACKUP: (
-                    self.keep_merge_backups,
-                    Defaults.merge_keep_backup,
-                ),
-                prefs.MERGETOOL: (self.mergetool, Defaults.mergetool),
-                prefs.RESIZE_BROWSER_COLUMNS: (
-                    self.resize_browser_columns,
-                    Defaults.resize_browser_columns,
-                ),
-                prefs.SPELL_CHECK: (self.check_spelling, Defaults.spellcheck),
-                prefs.MOUSE_ZOOM: (self.mouse_zoom, Defaults.mouse_zoom),
-            }
-        )
+        self.set_config({
+            prefs.SAVEWINDOWSETTINGS: (
+                self.save_window_settings,
+                Defaults.save_window_settings,
+            ),
+            prefs.TABWIDTH: (self.tabwidth, Defaults.tabwidth),
+            prefs.EXPANDTAB: (self.expandtab, Defaults.expandtab),
+            prefs.TEXTWIDTH: (self.textwidth, Defaults.textwidth),
+            prefs.LINEBREAK: (self.linebreak, Defaults.linebreak),
+            prefs.MAXRECENT: (self.maxrecent, Defaults.maxrecent),
+            prefs.SORT_BOOKMARKS: (self.sort_bookmarks, Defaults.sort_bookmarks),
+            prefs.DIFFTOOL: (self.difftool, Defaults.difftool),
+            prefs.EDITOR: (self.editor, fallback_editor()),
+            prefs.HISTORY_BROWSER: (
+                self.historybrowser,
+                prefs.default_history_browser(),
+            ),
+            prefs.BLAME_VIEWER: (self.blameviewer, Defaults.blame_viewer),
+            prefs.MERGE_KEEPBACKUP: (
+                self.keep_merge_backups,
+                Defaults.merge_keep_backup,
+            ),
+            prefs.MERGETOOL: (self.mergetool, Defaults.mergetool),
+            prefs.RESIZE_BROWSER_COLUMNS: (
+                self.resize_browser_columns,
+                Defaults.resize_browser_columns,
+            ),
+            prefs.SPELL_CHECK: (self.check_spelling, Defaults.spellcheck),
+            prefs.MOUSE_ZOOM: (self.mouse_zoom, Defaults.mouse_zoom),
+        })
 
         # pylint: disable=no-member
         self.fixed_font.currentFontChanged.connect(self.current_font_changed)
@@ -324,7 +347,8 @@ class AppearanceFormWidget(FormWidget):
     def __init__(self, context, model, parent):
         FormWidget.__init__(self, context, model, parent)
         # Theme selectors
-        self.theme = qtutils.combo_mapped(themes.options())
+        self.themes = themes.get_all_themes()
+        self.theme = qtutils.combo_mapped(themes.options(themes=self.themes))
         self.icon_theme = qtutils.combo_mapped(icons.icon_themes())
 
         # The transform to ustr is needed because the config reader will convert
@@ -345,20 +369,38 @@ class AppearanceFormWidget(FormWidget):
         self.add_row(N_('Indent Status paths'), self.status_indent)
         self.add_row(N_('Use a block cursor in diff editors'), self.block_cursor)
 
-        self.set_config(
-            {
-                prefs.BOLD_HEADERS: (self.bold_headers, Defaults.bold_headers),
-                prefs.HIDPI: (self.high_dpi, Defaults.hidpi),
-                prefs.STATUS_SHOW_TOTALS: (
-                    self.status_show_totals,
-                    Defaults.status_show_totals,
-                ),
-                prefs.STATUS_INDENT: (self.status_indent, Defaults.status_indent),
-                prefs.THEME: (self.theme, Defaults.theme),
-                prefs.ICON_THEME: (self.icon_theme, Defaults.icon_theme),
-                prefs.BLOCK_CURSOR: (self.block_cursor, Defaults.block_cursor),
-            }
-        )
+        self.set_config({
+            prefs.BOLD_HEADERS: (self.bold_headers, Defaults.bold_headers),
+            prefs.HIDPI: (self.high_dpi, Defaults.hidpi),
+            prefs.STATUS_SHOW_TOTALS: (
+                self.status_show_totals,
+                Defaults.status_show_totals,
+            ),
+            prefs.STATUS_INDENT: (self.status_indent, Defaults.status_indent),
+            prefs.THEME: (self.theme, Defaults.theme),
+            prefs.ICON_THEME: (self.icon_theme, Defaults.icon_theme),
+            prefs.BLOCK_CURSOR: (self.block_cursor, Defaults.block_cursor),
+        })
+
+        self.theme.currentIndexChanged.connect(self._theme_changed)
+
+    def _theme_changed(self, theme_idx):
+        """Set the icon theme to dark/light when the main theme changes"""
+        # Set the icon theme to a theme that corresponds to the main settings.
+        try:
+            theme = self.themes[theme_idx]
+        except IndexError:
+            return
+        icon_theme = self.icon_theme.current_data()
+        if theme.name == 'default':
+            if icon_theme in ('light', 'dark'):
+                self.icon_theme.set_value('default')
+        elif theme.is_dark:
+            if icon_theme in ('default', 'light'):
+                self.icon_theme.set_value('dark')
+        elif not theme.is_dark:
+            if icon_theme in ('default', 'dark'):
+                self.icon_theme.set_value('light')
 
 
 class AppearanceWidget(QtWidgets.QWidget):
