@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 import itertools
 import os
 from functools import partial
@@ -17,6 +16,7 @@ from ..settings import Settings
 from .. import actions
 from .. import cmds
 from .. import core
+from .. import difftool
 from .. import hotkeys
 from .. import icons
 from .. import qtutils
@@ -191,13 +191,14 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             cmds.RevertUnstagedEdits.name(),
             cmds.run(cmds.RevertUnstagedEdits, context),
             hotkeys.REVERT,
+            hotkeys.REVERT_ALT,
         )
         self.revert_unstaged_edits_action.setIcon(icons.undo())
 
         self.launch_difftool_action = qtutils.add_action(
             self,
-            cmds.LaunchDifftool.name(),
-            cmds.run(cmds.LaunchDifftool, context),
+            difftool.LaunchDifftool.name(),
+            cmds.run(difftool.LaunchDifftool, context),
             hotkeys.DIFF,
         )
         self.launch_difftool_action.setIcon(icons.diff())
@@ -504,7 +505,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                 cmds.do(cmds.StageModifiedAndUntracked, context)
                 return  # Nothing more to do.
             # At this point we may stage all modified and untracked, and then
-            # possibly a subset of the other category (eg. all modified and
+            # possibly a subset of the other category (e.g. all modified and
             # some untracked).  We don't return here so that StageOrUnstage
             # gets a chance to run below.
             if is_modified:
@@ -612,7 +613,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         self.copy_basename_action.setEnabled(enabled)
 
     def _set_staged(self, items):
-        """Adds items to the 'Staged' subtree."""
+        """Adds items to the 'Staged' sub-tree."""
         with qtutils.BlockSignals(self):
             self._set_subtree(
                 items,
@@ -623,7 +624,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             )
 
     def _set_modified(self, items):
-        """Adds items to the 'Modified' subtree."""
+        """Adds items to the 'Modified' sub-tree."""
         with qtutils.BlockSignals(self):
             self._set_subtree(
                 items,
@@ -633,7 +634,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             )
 
     def _set_unmerged(self, items):
-        """Adds items to the 'Unmerged' subtree."""
+        """Adds items to the 'Unmerged' sub-tree."""
         deleted_set = {path for path in items if not core.exists(path)}
         with qtutils.BlockSignals(self):
             self._set_subtree(
@@ -641,7 +642,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             )
 
     def _set_untracked(self, items):
-        """Adds items to the 'Untracked' subtree."""
+        """Adds items to the 'Untracked' sub-tree."""
         with qtutils.BlockSignals(self):
             self._set_subtree(items, UNTRACKED_IDX, N_('Untracked'), untracked=True)
 
@@ -667,7 +668,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         self._expand_items(idx, items)
 
         if prefs.status_show_totals(self.context):
-            parent.setText(0, '%s (%s)' % (parent_title, len(items)))
+            parent.setText(0, f'{parent_title} ({len(items)})')
 
     def _update_column_widths(self):
         self.resizeColumnToContents(0)
@@ -744,9 +745,8 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         widget = CopyLeadingPathWidget(
             N_('Copy Leading Path to Clipboard'), self.context, copy_menu
         )
-
-        # Store the value of the leading paths spinbox so that the value does not reset
-        # everytime the menu is shown and recreated.
+        # Store the value of the leading paths spin-box so that the value does not reset
+        # every time the menu is shown and recreated.
         widget.set_value(self.copy_leading_paths_value)
         widget.spinbox.valueChanged.connect(
             partial(setattr, self, 'copy_leading_paths_value')
@@ -1025,7 +1025,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         )
 
     def all_files(self):
-        """Return all of the current active files as a flast list"""
+        """Return all of the current active files as a flat list"""
         c = self.contents()
         return c.staged + c.unmerged + c.modified + c.untracked
 
@@ -1232,12 +1232,12 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
     def mousePressEvent(self, event):
         """Keep track of whether to drag URLs or just text"""
         self._alt_drag = event.modifiers() & Qt.AltModifier
-        return super(StatusTreeWidget, self).mousePressEvent(event)
+        return super().mousePressEvent(event)
 
     def mouseMoveEvent(self, event):
         """Keep track of whether to drag URLs or just text"""
         self._alt_drag = event.modifiers() & Qt.AltModifier
-        return super(StatusTreeWidget, self).mouseMoveEvent(event)
+        return super().mouseMoveEvent(event)
 
     def mimeData(self, items):
         """Return a list of absolute-path URLs"""
@@ -1247,7 +1247,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         return qtutils.mimedata_from_paths(context, paths, include_urls=include_urls)
 
     def mimeTypes(self):
-        """Return the mimetypes that this widget generates"""
+        """Return the mime types that this widget generates"""
         return qtutils.path_mimetypes(include_urls=not self._alt_drag)
 
 
@@ -1433,13 +1433,13 @@ class CustomizeCopyActions(standard.Dialog):
 
     def export_state(self):
         """Export the current state into the saved settings"""
-        state = super(CustomizeCopyActions, self).export_state()
+        state = super().export_state()
         standard.export_header_columns(self.table, state)
         return state
 
     def apply_state(self, state):
         """Restore state from the saved settings"""
-        result = super(CustomizeCopyActions, self).apply_state(state)
+        result = super().apply_state(state)
         standard.apply_header_columns(self.table, state)
         return result
 
@@ -1571,7 +1571,7 @@ def _transplant_selection_across_sections(
     modified_paths = modified[NEW_PATHS_IDX]
     untracked_paths = untracked[NEW_PATHS_IDX]
 
-    # These callbacks select a path in the corresponding widget subtree lists.
+    # These callbacks select a path in the corresponding widget sub-tree lists.
     select_staged = staged[SELECT_FN_IDX]
     select_unmerged = unmerged[SELECT_FN_IDX]
     select_modified = modified[SELECT_FN_IDX]
@@ -1635,7 +1635,7 @@ def _transplant_selection_across_sections(
 
 
 class CopyLeadingPathWidget(QtWidgets.QWidget):
-    """A widget that holds a label and a spinbox for the number of paths to strip"""
+    """A widget that holds a label and a spin-box for the number of paths to strip"""
 
     def __init__(self, title, context, parent):
         QtWidgets.QWidget.__init__(self, parent)
@@ -1666,35 +1666,35 @@ class CopyLeadingPathWidget(QtWidgets.QWidget):
         disabled_text_rgb = theme.disabled_text_color_rgb()
 
         stylesheet = """
-            * {
+            * {{
                 show-decoration-selected: 1
-            }
-            QLabel {
-                color: %(text_rgb)s;
+            }}
+            QLabel {{
+                color: {text_rgb};
                 show-decoration-selected: 1
-            }
-            QLabel:hover {
-                color: %(highlight_text_rgb)s;
-                background-color: %(highlight_rgb)s;
+            }}
+            QLabel:hover {{
+                color: {highlight_text_rgb};
+                background-color: {highlight_rgb};
                 background-clip: padding;
                 show-decoration-selected: 1
-            }
-            QLabel:disabled {
-                color: %(disabled_text_rgb)s;
-            }
-        """ % {
-            'disabled_text_rgb': disabled_text_rgb,
-            'text_rgb': text_rgb,
-            'highlight_text_rgb': highlight_text_rgb,
-            'highlight_rgb': highlight_rgb,
-        }
+            }}
+            QLabel:disabled {{
+                color: {disabled_text_rgb};
+            }}
+        """.format(
+            disabled_text_rgb=disabled_text_rgb,
+            text_rgb=text_rgb,
+            highlight_text_rgb=highlight_text_rgb,
+            highlight_rgb=highlight_rgb,
+        )
 
         self.setStyleSheet(stylesheet)
 
     def value(self):
-        """Return the current value of the spinbox"""
+        """Return the current value of the spin-box"""
         return self.spinbox.value()
 
     def set_value(self, value):
-        """Set the spinbox value"""
+        """Set the spin-box value"""
         self.spinbox.setValue(value)
